﻿using System;
using System.Collections.Generic;
using System.IO;
using UnityEngine;
using HarmonyLib;
using System.Reflection;
using UnityEngine.SceneManagement;
using System.Linq;

public class LargerNets : Mod
{
    private Harmony harmony;
    private readonly string harmonyID = "com.raftmodding.marsh.largernet";
    private readonly string dataPath = @"\mods\ModData\LargerNets\amount.txt";
    private readonly int defaultNumber = 50; //This is larger nets default amount

    public static LargerNets _instance;
    public int amount;

    private bool isLoaded;
    private static string prefix = "[" + "<color=#0000ff>LargerNets</color>" + "] ";

    private void Start()
    {
        _instance = this;
        amount = defaultNumber;

        harmony = new Harmony(harmonyID);
        harmony.PatchAll(Assembly.GetExecutingAssembly());


        //Just clearing the player prefs from old users
        if (PlayerPrefs.HasKey("largernets_amount"))
        {
            PlayerPrefs.DeleteKey("largernets_amount");
        }

        //Setting the amount from what has been saved before
        if (File.Exists(Directory.GetCurrentDirectory() + dataPath))
        {
            string[] contents = File.ReadAllLines(Directory.GetCurrentDirectory() + dataPath);
            if (contents.Length >= 1)
            {
                int result = defaultNumber;
                if (int.TryParse(contents[0], out result))
                {
                    amount = result;
                }
                else
                {
                    Debug.LogError(prefix + " Failed to convert \"" + contents[0] + "\" to a int, please check the data file");
                    Debug.LogWarning(prefix + " Using LargerNet's default value of " + defaultNumber);
                }
            }
        }

        Debug.Log(prefix + " LargerNets loaded!");
    }
    private void ChangeValue(string[] args)
    {
        if (args.Length == 0)
        {
            Debug.Log(prefix + "Error, example \nnetamount 50");
            return;
        }
        if (SceneManager.GetActiveScene().buildIndex != 1)
        {
            Debug.Log(prefix + "You need to be in game to use this command");
            return;
        }
        int result = 10;
        if (int.TryParse(args[0], out result))
        {
            Traverse.Create(FindObjectOfType<ItemCollector>()).Field("maxNumberOfItems").SetValue(result);
            amount = result;
            Debug.Log(prefix + " Set max items of nets to " + amount);
        }
    }

    public void OnModUnload()
    {
        try
        {
            if (!Directory.Exists(Directory.GetCurrentDirectory() + @"\mods\ModData\LargerNets"))
                Directory.CreateDirectory(Directory.GetCurrentDirectory() + @"\mods\ModData\LargerNets");
            File.WriteAllText(Directory.GetCurrentDirectory() + dataPath, amount + "");
        }
        catch (Exception e)
        {
            Debug.LogError(prefix + " Failed saving the amount \n" + e.ToString());
        }

        harmony.UnpatchAll(harmonyID);
        if (SceneManager.GetActiveScene().buildIndex == 1)
        {
            ItemCollector[] collectors = FindObjectsOfType<ItemCollector>();
            foreach (ItemCollector collector in collectors)
            {
                Traverse.Create(collector).Field("maxNumberOfItems").SetValue(10);
            }
            Debug.Log(prefix + " Set all of the nets back to the default value");
        }
        Debug.Log(prefix + " Unloaded!");
        Destroy(gameObject);
    }

    [ConsoleCommand(name: "netamount", docs: "Changes the amount of items the item nets can hold")]
    public static void netamountCommand(string[] args)
    {
        _instance.ChangeValue(args);
    }
}

[HarmonyPatch(typeof(ItemCollector))]
[HarmonyPatch("OnTriggerEnter")]
class OnTriggerEnterPatch
{
    static void Prefix(ItemCollector __instance)
    {
        //On Item Picked up, it checks the value stored in the static script of how many the max should be and it sets the private variable to that
        //On Unload it calls a custom method on all of them to reset them.
        int value = (int)Traverse.Create(__instance).Field("maxNumberOfItems").GetValue();
        if (value != LargerNets._instance.amount)
        {
            Traverse.Create(__instance).Field("maxNumberOfItems").SetValue(LargerNets._instance.amount);
        }
    }
}
